@@ -41,9 +41,10 @@ use LWP::UserAgent;
 use HTTP::Request;
 use URI;
 use Scalar::Util 'blessed';
+use File::Temp;
 
 use RPC::XML;
-require RPC::XML::Parser;
+require RPC::XML::ParserFactory;
 
 BEGIN
 {
@@ -52,7 +53,8 @@ BEGIN
     $COMPRESSION_AVAILABLE = ($@) ? '' : 'deflate';
 }
 
-$VERSION = '1.29';
+$VERSION = '1.31';
+$VERSION = eval $VERSION; ## no critic
 
 ###############################################################################
 #
@@ -140,10 +142,10 @@ sub new
         delete $attrs{error_handler};
     }
 
-    # Get the RPC::XML::Parser instance
-    $self->{__parser} = RPC::XML::Parser->new($attrs{parser} ?
-                                              @{$attrs{parser}} : ()) or
-        return "${class}::new: Unable to get RPC::XML::Parser object";
+    # Get the RPC::XML::Parser instance from the ParserFactory
+    $self->{__parser} =
+        RPC::XML::ParserFactory->new($attrs{parser} ? @{$attrs{parser}} : ())
+              or return "${class}::new: Unable to get RPC::XML::Parser object";
     delete $attrs{parser};
 
     # Now preserve any remaining attributes passed in
@@ -182,7 +184,7 @@ sub simple_request
     unless (ref $return)
     {
         $RPC::XML::ERROR = ref($self) . "::simple_request: $return";
-        return undef;
+        return;
     }
     $return->value;
 }
@@ -208,7 +210,7 @@ sub send_request
     my ($self, $req, @args) = @_;
 
     my ($me, $message, $response, $reqclone, $content, $can_compress, $value,
-        $do_compress, $req_fh, $tmpfile, $com_engine);
+        $do_compress, $req_fh, $tmpdir, $com_engine);
 
     $me = ref($self) . '::send_request';
 
@@ -241,13 +243,11 @@ sub send_request
         require File::Spec;
         require Symbol;
         # Start by creating a temp-file
-        $tmpfile = $self->message_temp_dir || File::Spec->tmpdir;
-        ($tmpfile = File::Spec->catfile($tmpfile, __PACKAGE__ . $$ . time)) =~
-            s/::/-/g; # Colons in filenames bad on some systems!
+        $tmpdir = $self->message_temp_dir || File::Spec->tmpdir;
         $req_fh = Symbol::gensym();
-        return "$me: Error opening $tmpfile: $!"
-            unless (open($req_fh, "+> $tmpfile"));
-        unlink $tmpfile;
+        return "$me: Error opening tmpfile: $!"
+             unless ($req_fh = File::Temp->new(UNLINK=>1, DIR=>$tmpdir));
+        binmode($req_fh);
         # Make it auto-flush
         my $old_fh = select($req_fh); $| = 1; select($old_fh);
 
@@ -259,10 +259,8 @@ sub send_request
         if ($do_compress && ($req->length >= $self->compress_thresh))
         {
             my $fh2 = Symbol::gensym();
-            $tmpfile .= '-2';
-            return "$me: Error opening $tmpfile: $!"
-                unless (open($fh2, "+> $tmpfile"));
-            unlink $tmpfile;
+            return "$me: Error opening tmpfile: $!"
+                unless ($fh2 = File::Temp->new(UNLINK=>1, DIR=>$tmpdir));
             # Make it auto-flush
             $old_fh = select($fh2); $| = 1; select($old_fh);
 
@@ -304,8 +302,7 @@ sub send_request
         $reqclone->content_length(-s $req_fh);
         $reqclone->content(sub {
                                my $b = '';
-                               return undef
-                                   unless defined(read($req_fh, $b, 4096));
+                               return unless defined(read($req_fh, $b, 4096));
                                $b;
                            });
     }
@@ -463,7 +460,7 @@ sub credentials
 # Immutable accessor methods
 BEGIN
 {
-    no strict 'refs';
+    no strict 'refs'; ## no critic
 
     for my $method (qw(useragent request compress_re compress parser))
     {
@@ -593,11 +590,11 @@ treated specially:
 
 =item parser
 
-If this parameter is passed, the value following it is expected to be an
-array reference. The contents of that array are passed to the B<new> method
-of the B<RPC::XML::Parser> object that the client object caches for its use.
-See the B<RPC::XML::Parser> manual page for a list of recognized parameters
-to the constructor.
+If this parameter is passed, the value following it is expected to be an array
+reference. The contents of that array are passed to the B<new> method of the
+B<RPC::XML::ParserFactory>-generated object that the client object caches for
+its use. See the B<RPC::XML::ParserFactory> manual page for a list of
+recognized parameters to the constructor.
 
 =item useragent
 
@@ -641,7 +638,7 @@ temporary file, and spooled from there instead. This is useful for cases in
 which the request includes B<RPC::XML::base64> objects that are themselves
 spooled from file-handles. This test is independent of compression, so even
 if compression of a request would drop it below this threshhold, it will be
-spooled anyway. The file itself is unlinked after the file-handle is created,
+spooled anyway. The file itself is created via File::Temp with UNLINK=>1,
 so once it is freed the disk space is immediately freed.
 
 =item message_temp_dir
@@ -846,6 +843,6 @@ L<RPC::XML>, L<RPC::XML::Server>
 
 =head1 AUTHOR
 
-Randy J. Ray <rjray@blackperl.com>
+Randy J. Ray C<< <rjray@blackperl.com> >>
 
 =cut
